#[macro_export]
/// Macro to support extraction of subsets of JSON object fields.
///
/// Optionally `extract!` accepts a `Utf8Buffer` to use when
/// decoding object keys. Also `decoder` can be omitted in which
/// case the macro expands to a lambda function expecting the
/// decoder as argument (this form is useful for nested `object!`
/// invocations).
///
/// # Example:
///
/// ```text
/// let p = extract! {
///     let decoder = d;
///     x: req. u32 = "x" => Decoder::from_json,
///     y: req. u32 = "y" => Decoder::from_json
/// }
/// println!("x={}, y={}", p.x, p.y)
/// ```
macro_rules! extract {

    // expression form with decoder and buffer
    (
      let decoder = $dec:ident;
      let buffer  = $buf:expr;
      $($name:ident: $modus:ident. $typ:ty = $key:expr => $action:expr),+
    ) => {{
        struct X {
            $($name: $typ),+
        }
        object! {
            let decoder = $dec;
            let buffer  = &mut $buf;
            X {
                $($name: $modus. $key => $action),+
            }
        }
    }};

    // lambda form with buffer
    (
      let buffer = $buf:expr;
      $($name:ident: $modus:ident. $typ:ty = $key:expr => $action:expr),+
    ) => {{ |dec| {
        struct X {
            $($name: $typ),+
        }
        object! {
            let decoder = dec;
            let buffer  = &mut $buf;
            X {
                $($name: $modus. $key => $action),+
            }
        }

    }}};

    // expression form with decoder using `String`s for keys
    (
      let decoder = $dec:ident;
      $($name:ident: $modus:ident. $typ:ty = $key:expr => $action:expr),+
    ) => {{
        struct X {
            $($name: $typ),+
        }
        object! {
            let decoder = $dec;
            X {
                $($name: $modus. $key => $action),+
            }
        }
    }};

    // lambda form using `String`s for keys
    (
      $($name:ident: $modus:ident. $typ:ty = $key:expr => $action:expr),+
    ) => {{ |dec| {
        struct X {
            $($name: $typ),+
        }
        object! {
            let decoder = dec;
            X {
                $($name: $modus. $key => $action),+
            }
        }
    }}}
}

#[macro_export]
/// Macro to support declarative decoding into struct types.
///
/// Optionally `object!` accepts a `Utf8Buffer` to use when
/// decoding object keys. Also `decoder` can be omitted in which
/// case the macro expands to a lambda function expecting the
/// decoder as argument (this form is useful for nested `object!`
/// invocations).
///
/// # Example:
///
/// ```text
/// struct Point {
///     x: u32,
///     y: u32
/// }
///
/// object! {
///     let decoder = d;
///     Point {
///         x: req. "x" => Decoder::u32,
///         y: req. "y" => Decoder::u32
///     }
/// }
/// ```
macro_rules! object {

    // expression form with decoder and buffer
    (
      let decoder = $dec:ident;
      let buffer  = $buf:expr;
      $T:ident {
          $($name:ident: $modus:ident. $key:expr => $action:expr),+
      }
    ) => {{
        $dec.object1(|$dec| {
            $(let mut $name = None;)+
            loop {
                $buf.reset();
                try!($dec.key_str($buf, false));
                match $buf.as_str() {
                    $($key => {
                        let v = try!($dec.value($action));
                        $name = Some(v.value);
                        if v.is_last {
                            break
                        }
                    })+
                    _ => {
                        let v = try!($dec.value($crate::Decoder::skip));
                        if v.is_last {
                            break
                        }
                    }
                }
            }
            Ok($T {
                $($name: to_field!($key, $name, $modus),)+
            })
        })
    }};

    // lambda form with buffer
    (
      let buffer = $buf:expr;
      $T:ident {
          $($name:ident: $modus:ident. $key:expr => $action:expr),+
      }
    ) => {{ |dec| {
        dec.object1(|dec| {
            $(let mut $name = None;)+
            loop {
                $buf.reset();
                try!(dec.key_str($buf, false));
                match $buf.as_str() {
                    $($key => {
                        let v = try!(dec.value($action));
                        $name = Some(v.value);
                        if v.is_last {
                            break
                        }
                    })+
                    _ => {
                        let v = try!(dec.value($crate::Decoder::skip));
                        if v.is_last {
                            break
                        }
                    }
                }
            }
            Ok($T {
                $($name: to_field!($key, $name, $modus),)+
            })
        })
    }}};

    // expression form with decoder using `String`s for keys
    (
      let decoder = $dec:ident;
      $T:ident {
          $($name:ident: $modus:ident. $key:expr => $action:expr),+
      }
    ) => {{
        $dec.object1(|$dec| {
            $(let mut $name = None;)+
            loop {
                match try!($dec.key()).as_ref() {
                    $($key => {
                        let v = try!($dec.value($action));
                        $name = Some(v.value);
                        if v.is_last {
                            break
                        }
                    })+
                    _ => {
                        let v = try!($dec.value($crate::Decoder::skip));
                        if v.is_last {
                            break
                        }
                    }
                }
            }
            Ok($T {
                $($name: to_field!($key, $name, $modus),)+
            })
        })
    }};

    // lambda form using `String`s for keys
    (
      $T:ident {
          $($name:ident: $modus:ident. $key:expr => $action:expr),+
      }
    ) => {{ |dec| {
        dec.object1(|dec| {
            $(let mut $name = None;)+
            loop {
                match try!(dec.key()).as_ref() {
                    $($key => {
                        let v = try!(dec.value($action));
                        $name = Some(v.value);
                        if v.is_last {
                            break
                        }
                    })+
                    _ => {
                        let v = try!(dec.value($crate::Decoder::skip));
                        if v.is_last {
                            break
                        }
                    }
                }
            }
            Ok($T {
                $($name: to_field!($key, $name, $modus),)+
            })
        })
    }}};
}

#[macro_export]
macro_rules! to_field {
    ($msg:expr, $e:expr, opt) => {
        match $e {
            Some(e) => e,
            None    => None
        }
    };
    ($msg:expr, $e:expr, req) => {
        match $e {
            Some(e) => e,
            None    => return Err($crate::DecodeError::Expected($msg))
        }
    }
}

#[macro_export]
/// Macro to support declarative decoding into `Vec`.
///
/// This only supports homogenous JSON arrays. To decode
/// arrays with different element types use `Decoder::array`.
///
/// # Syntax
///
/// ```text
/// array!(decoder_var_name ',' expression) /
/// array!(expression)
/// ```
/// Two variants are supported, one with an explicit decoder
/// variable and the other one without. If not given, a lambda
/// function will be generated which needs to be supplied to a
/// decoder and is best used inside `object!` macro invications.
macro_rules! array {
    ($dec:ident, $e:expr) => {{
        let mut v = Vec::new();
        for x in try!($dec.array_iter($e)) {
            v.push(try!(x))
        }
        Ok(v)
    }};
    ($e:expr) => {{ |dec| {
        let mut v = Vec::new();
        for x in try!(dec.array_iter($e)) {
            v.push(try!(x))
        }
        Ok(v)
    }}}
}

